/*
 * This file is part of the QPackageKit project
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 * Copyright (C) 2010-2018 Daniel Nicoletti <dantti12@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB. If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#define AUTH_PACKAGE_INSTALL			"org.freedesktop.packagekit.package-install"
#define AUTH_PACKAGE_INSTALL_UNTRUSTED		"org.freedesktop.packagekit.package-install-untrusted"
#define AUTH_SYSTEM_TRUST_SIGNING_KEY		"org.freedesktop.packagekit.system-trust-signing-key"
#define AUTH_PACKAGE_EULA_ACCEPT		"org.freedesktop.packagekit.package-eula-accept"
#define AUTH_PACKAGE_REMOVE			"org.freedesktop.packagekit.package-remove"
#define AUTH_SYSTEM_UPDATE			"org.freedesktop.packagekit.system-update"
#define AUTH_SYSTEM_SOURCES_CONFIGURE		"org.freedesktop.packagekit.system-sources-configure"
#define AUTH_SYSTEM_SOURCES_REFRESH		"org.freedesktop.packagekit.system-sources-refresh"
#define AUTH_SYSTEM_NETWORK_PROXY_CONFIGURE	"org.freedesktop.packagekit.system-network-proxy-configure"
#define AUTH_CANCEL_FOREIGN 			"org.freedesktop.packagekit.cancel-foreign"
